@@ -49,7 +49,7 @@ static int win_fd;
 static int pty_fd = -1;
 static int log_fd = -1;
 #if CYGWIN_VERSION_API_MINOR >= 74
-static struct winsize prev_winsize = (struct winsize){0, 0, 0, 0};
+static struct winsize prev_winsize = {0, 0, 0, 0};
 #else
 static struct winsize prev_winsize;
 #endif
@@ -87,7 +87,7 @@ childerror(char * action, bool from_fork, int errno_code, int code)
 
   char s[33];
   bool colour_code = code && !errno_code;
-  sprintf(s, "\033[30;%dm\033[K", from_fork ? 41 : colour_code ? code : 43);
+  sprintf(s, "\r\n\033[30;%dm\033[K", from_fork ? 41 : colour_code ? code : 43);
   term_write(s, strlen(s));
   term_write(action, strlen(action));
   if (errno_code) {
@@ -278,6 +278,22 @@ trim_environment(void)
   trimenv("BYOBU_");
 }
 
+static bool
+ispathprefix(string pref, string path)
+{
+  if (*pref == '/')
+    pref++;
+  if (*path == '/')
+    path++;
+  int len = strlen(pref);
+  if (0 == strncmp(pref, path, len)) {
+    path += len;
+    if (!*path || *path == '/')
+      return true;
+  }
+  return false;
+}
+
 void
 child_create(char *argv[], struct winsize *winp)
 {
@@ -294,6 +310,19 @@ child_create(char *argv[], struct winsize *winp)
   signal(SIGTERM, sigexit);
   signal(SIGQUIT, sigexit);
 
+  // support OSC 7 directory cloning if cloning WSL window while in rootfs
+  if (support_wsl && wslname) {
+    // this is done once in a new window, so don't care about memory leaks
+    char * rootfs = path_win_w_to_posix(wsl_basepath);
+    char * cwd = getcwd(malloc(MAX_PATH), MAX_PATH);
+    if (ispathprefix(rootfs, cwd)) {
+      char * dist = cs__wcstombs(wslname);
+      char * wslsubdir = cwd + strlen(rootfs);
+      char * wsldir = asform("//wsl$/%s%s", dist, wslsubdir);
+      chdir(wsldir);
+    }
+  }
+
   // Create the child process and pseudo terminal.
   pid = forkpty(&pty_fd, 0, 0, winp);
   if (pid < 0) {
@@ -472,6 +501,19 @@ child_tty(void)
 
 #define patch_319
 
+#ifdef debug_pty
+static void
+trace_line(char * tag, int n, const char * s, int len)
+{
+  printf("%s %d", tag, n);
+  for (int i = 0; i < len; i++)
+    printf(" %02X", (uint)s[i]);
+  printf("\n");
+}
+#else
+#define trace_line(tag, n, s, len)	
+#endif
+
 void
 child_proc(void)
 {
@@ -605,6 +647,7 @@ child_proc(void)
 #endif
         do {
           int ret = read(pty_fd, buf + len, sizeof buf - len);
+          trace_line("read", ret, buf + len, ret);
           //if (kb_trace) printf("[%lu] read %d\n", mtime(), ret);
           if (ret > 0)
             len += ret;
@@ -614,6 +657,8 @@ child_proc(void)
 
         if (len > 0) {
           term_write(buf, len);
+          trace_line("twrt", len, buf, len);
+
           // accelerate keyboard echo if (unechoed) keyboard input is pending
           if (kb_input) {
             kb_input = false;
@@ -811,8 +856,13 @@ grandchild_process_list(void)
 void
 child_write(const char *buf, uint len)
 {
-  if (pty_fd >= 0)
+  if (pty_fd >= 0) {
+#ifdef debug_pty
+    int n =
+#endif
     write(pty_fd, buf, len);
+    trace_line("cwrt", n, buf, len);
+  }
 }
 
 /*
@@ -840,8 +890,13 @@ child_printf(const char *fmt, ...)
     char *s;
     int len = vasprintf(&s, fmt, va);
     va_end(va);
-    if (len >= 0)
+    if (len >= 0) {
+#ifdef debug_pty
+      int n =
+#endif
       write(pty_fd, s, len);
+      trace_line("tprt", n, s, len);
+    }
     free(s);
   }
 }
