@@ -1,5 +1,5 @@
 // termclip.c (part of mintty)
-// Copyright 2008-10 Andy Koppe, 2018 Thomas Wolff
+// Copyright 2008-23 Andy Koppe, 2018 Thomas Wolff
 // Adapted from code from PuTTY-0.60 by Simon Tatham and team.
 // Licensed under the terms of the GNU General Public License v3 or later.
 
@@ -94,6 +94,8 @@ clip_addchar(clip_workbuf * b, wchar chr, cattr * ca, bool tabs, ulong sizehint)
 static clip_workbuf *
 get_selection(bool attrs, pos start, pos end, bool rect, bool allinline, bool with_tabs)
 {
+  //printf("get_selection attrs %d all %d tabs %d\n", attrs, allinline, with_tabs);
+
   clip_workbuf *buf = newn(clip_workbuf, 1);
   *buf = (clip_workbuf){.with_attrs = attrs,
                         .capacity = 0, .len = 0, .text = 0, .cattrs = 0};
@@ -114,8 +116,13 @@ get_selection(bool attrs, pos start, pos end, bool rect, bool allinline, bool wi
     bool nl = false;
     termline *line = fetch_line(start.y);
 
-    if (start.y == term.curs.y) {
-      line->chars[term.curs.x].attr.attr |= TATTR_ACTCURS;
+    if (allinline) {
+      // this tweak (commit 975403 "export HTML: consider cursor", 2.9.1)
+      // causes cursor artefacts in connection with ClicksPlaceCursor=yes
+      // now guarded to cases of HTML copy/export
+      if (start.y == term.curs.y) {
+        line->chars[term.curs.x].attr.attr |= TATTR_ACTCURS;
+      }
     }
 
     pos nlpos;
@@ -265,15 +272,11 @@ term_open(void)
   wchar * p = selstr;
   while (iswspace(*p))
     p++;
-  if (*p) {
-    wchar * url = p;
-    while (*p && !iswspace(*p))
-      p++;
-    *p = 0;
-    win_open(wcsdup(url), true);  // win_open frees its argument
-  }
 
-  free(selstr);
+  if (*p)
+    win_open(selstr, true);  // win_open frees its argument
+  else
+    free(selstr);
 }
 
 static bool
